/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.33.00 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Hrushikesh Vasuki 07/29/05  First release of source file
* Hrushikesh Vasuki 10/07/05  Mods to increase filter block size to 256
*                             samples and use 30F6014A device
*                             instead of 30F6014
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
*
**********************************************************************/

#include "p30fxxxx.h"
#include "common.h"
#include "dsp.h"

//Macros for Configuration Fuse Registers:
//Invoke macros to set up  device configuration fuse registers.
//The fuses will select the oscillator source, power-up timers, watch-dog
//timers, BOR characteristics etc. The macros are defined within the device
//header files. The configuration fuse registers reside in Flash memory.
_FOSC(CSW_FSCM_OFF & XT_PLL16);  //Run this project using an external crystal
                                //routed via the PLL in 16x multiplier mode
                                //For the 7.3728 MHz crystal we will derive a
                                //throughput of 7.3728e+6*16/4 = 29.4 MIPS(Fcy)
                                //,~33.9 nanoseconds instruction cycle time(Tcy).
_FWDT(WDT_OFF);                 //Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_OFF);   //Enable MCLR reset pin and turn off the
                                //power-up timers.
_FGS(CODE_PROT_OFF);            //Disable Code Protection

extern IIRTransposedStruct ExampleHPFFilter;
extern void IIRTransposedInit ( IIRTransposedStruct* filter );
extern fractional* IIRTransposed (      int numSamps,
                                        fractional* dstSamps,
                                        fractional* srcSamps,
                                        IIRTransposedStruct* filter
                                );
extern IIRTransposedStruct ExampleHPFFilter;

fractional inputSignal[NUMSAMP];
fractional outputSignal[NUMSAMP];
fractional* oPtr;
fractional* iPtr;
unsigned int doFilterFlag;

int main (void);

int main (void)
{
        iPtr = &inputSignal[0];
        doFilterFlag = 0;
        IIRTransposedInit (&ExampleHPFFilter);  //Initialize state variables
                                //in the digital filter to zero. This is
                                //done so that arbitrary values in RAM are not
                                //treated as state variables for the filter
                                //on any reset
        ADC_Init();             //Initialize the A/D converter

        while (1)               //Loop Endlessly - Execution is interrupt driven
        {                       //from this point on.
                if (doFilterFlag)
                {
                        //Filter a block of NUMSAMP samples using the IIR transposed canonical biquad
                        oPtr = IIRTransposed(NUMSAMP, &outputSignal[0], &inputSignal[0], &ExampleHPFFilter);
                        doFilterFlag = 0;
                        iPtr = &inputSignal[0];
                }
        }
        return 0;
}






