
/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        isr_INTpin.c
* Dependencies:    p30fxxxx.h
* Processor:       dsPIC30F6014
* Compiler:        MPLAB C30 v1.32.00 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Richard Fischer   07/18/05  INTx pin interrupt handler
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
*
**********************************************************************/

#include <p30fxxxx.h>
#include "LCD Display.h"
#include "common.h"



/*---------------------------------------------------------------------
  Function Name: INT1Interrupt
  Description:   INT1 pin Interrupt Handler
  Inputs:        None
  Returns:       None
-----------------------------------------------------------------------*/
void _ISR _INT1Interrupt( void )
{
	unsigned int temp_delay = 0x8FFF;
	
	/* crude delay for debounce, may need to adjust */
	while ( temp_delay --);   // crude short delay

	unsigned char valid_key = 0;
	
	if ( !PORTAbits.RA13 ) {
		valid_key = 1;
	}
	else if ( !PORTAbits.RA14 ) {
		valid_key = 2;
	}
	else if ( !PORTAbits.RA15 ) {
		valid_key = 3;
	}
	
	switch ( valid_key )               // evaluate expression 
	{
		case ( 1 ):                    // test if hours will be incremented
			hours++;                   // increment hours
			if ( hours > 23 )          // test if hours will roll over
			{
				hours = 0x00;          // yes, so reset hours
			}
			break;                     // exit from switch evaluation
			
		case ( 2 ):                    // test if minutes will be incremented
			minutes++;                 // increment minutes
			if ( minutes > 59  )       // test if minutes will roll over
			{
				minutes = 0x00;        // yes, so reset minutes
			}
			break;                     // exit from switch evaluation
 
		case ( 3 ):                    // test if seconds will be incremented
			seconds++;                 // increment seconds
			if ( seconds > 59 )        // test if seconds will roll over
			{
				seconds = 0x00;        // yes, so reset seconds
			}
			break;                     // exit from switch evaluation
	
		default:                       // if no match occurs
			break;                     // exit from switch evaluation
	}	

	valid_key = 0;

	/* reset interrupt flag */
	IFS1bits.INT1IF = 0;    
}	
