/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        address_error_trap.s
* Dependencies:    Header (.h/.inc) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.21 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV                09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* The user should check to ensure that the PC value retrieved from
* the stack is a valid program memory address. IF it isn't then
* the user may conclude that the address error trap was generated
* because an invalid value was pushed into the stack and followed by a
* a "RETFIE" or "RETURN" instruction.
**********************************************************************/

.include "p30f6014a.inc"

.global __AddressError
.global _AddrErrInstructionAddrLow, _AddrErrInstructionAddrHigh

.section .bss
_AddrErrInstructionAddrLow: .space 2
_AddrErrInstructionAddrHigh: .space 2

.section .text

__AddressError:
        mov     #_AddrErrInstructionAddrHigh, w1
        pop     [w1--]                  ;Pop the Program Counter (PC) from the stack
        pop     [w1++]                  ;Remember that the PC(PCH and PCL) is stacked
        bclr    [w1], #7                ;along with the SRL byte and IPL3 bit from
        inc     w1, w1                  ;CORCON. So we need to extract just the 24-bit
        clr.b   [w1]                    ;PCH:PCL information from the stack
        mov     #_AddrErrInstructionAddrLow, w1  ;Decrement that value by 2
        mov     #2, w2                  ;Store the 24-bit result into 2x16-bit words
        subr    w2, [w1], [w1++]        ;in RAM at- AddrErrInstructionAddrHigh:Low
        clr     w2
        subbr   w2, [w1], [w1]
        bclr    INTCON1, #ADDRERR       ;Clear the trap flag
StayTrappedAddrErr:                     ;Stay in this routine
        bra     StayTrappedAddrErr
        ;Place a breakpoint above and halt the Debugging process and view
        ;the variables named: AddrErrInstructionAddrHigh:Low
        retfie

/*
EOF
*/


