/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        stack_error_trap.s
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.32.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
**********************************************************************/

.include "p30f6014a.inc"

.global __StackError
.global _StkOvflErrInstructionAddrLow, _StkOvflErrInstructionAddrHigh



.section .bss
_StkOvflErrInstructionAddrLow: 	.space 2
_StkOvflErrInstructionAddrHigh: .space 2

.section .text

__StackError:
	mov	w15, w0		;Load w0 with the stack pointer value
	mov	SPLIM, w1	;Load w1 with the SPLIM value

StackOverflowCheck:
	cp	w0, w1
	bra	lt, StackUnderflow
	mov	#_StkOvflErrInstructionAddrHigh, w1
	pop	[w1--]		;Pop the PCL and PCH from the stack
	pop	[w1++]		;Decrement that value by 2
	bclr	[w1], #7	;Store the result into RAM at:
	inc	w1, w1		;OffendingInstructionAddrHigh:Low
	clr.b	[w1]		;Halt the Debugging process and view
				;the variables named:
				;OffendingInstructionAddrHigh:Low
	bclr	INTCON1, #STKERR ;Clear the trap flag
StkOverflowErr:			;Stay in this routine
	bra	StkOverflowErr

StackUnderflow:
	bclr	INTCON1, #STKERR ;Clear the trap flag
StkUnderflowErr:		;Stay in this routine
	bra	StkUnderflowErr	;To find the offending instruction that caused
				;the Stack underflow error use MPLAB SIM
	retfie


.end

/*
EOF
*/


