/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        math_error_trap_30F_GP_MC_Sensor.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.32.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* This routine applies for all dsPIC30F Motor Control, General Purpose
* and Sensor family devices.
* NOTE: Disable compiler optimizations for this file
**********************************************************************/

#include "p30fxxxx.h"
void __attribute__((__interrupt__)) _MathError(void);

unsigned char   accAOvfl = 0;
unsigned char   accBOvfl = 0;
unsigned char   divByZero = 0;
unsigned int    MathErrInstructionAddr[2];

//_MathErr() is the Arithmetic Error Trap handler.
//The routine must have global scope in order to be an ISR.
//The trap handler name is chosen from the device linker script.
//The code execution reaches here based on Accumulator overflow errors
//or divide by zero errors
void __attribute__((__interrupt__)) _MathError(void)
{
        if (SRbits.OA && (INTCON1bits.OVATE || INTCON1bits.COVTE) )
        {
                ACCA.L = 0;
                ACCA.H = 0;     /* Before clearing the MATHERR flag, */
                ACCA.U = 0;     /* we should clear the condition causing the trap */
                SRbits.OA = 0;  /* This is true of Math errors caused by Accumulator */
                SRbits.OAB = 0; /* overflows only */
                accAOvfl = 0xFF;
        }
        else if (SRbits.OB && (INTCON1bits.OVBTE || INTCON1bits.COVTE) )
        {
                ACCB.L = 0;
                ACCB.H = 0;
                ACCB.U = 0;
                SRbits.OB = 0;
                SRbits.OAB = 0;
                accBOvfl = 0xFF;
        }
        else
        {
                divByZero = 0xFF;
        }
        INTCON1bits.MATHERR = 0;
        asm volatile (" push.d   w0 \n"
                        "push     w2 \n"
                        "mov    w15, w0 \n"
                        "sub    w0, #(6+4), w0 \n"           /*NOTE: Subtracting 6 to estimate stack pointer*/
                                                             /*location prior to 2 push ops*/
                                                             /*Subtracting an additional 4 to compensate for compiler*/
                                                             /*pushing 2 words at the beginning of the trap handler*/
                        "mov     #_MathErrInstructionAddr+2, w1 \n"
                        "mov     [--w0], [w1--] \n"          /*Pop the Program Counter (PC) from the stack */
                        "mov     [--w0], [w1++] \n"          /*Remember that the PC(PCH and PCL) is stacked  */
                        "bclr    [w1], #7 \n"                /*along with the SRL byte and IPL3 bit from */
                        "inc     w1, w1 \n"                  /*CORCON. So we need to extract just the 24-bit */
                        "clr.b   [w1] \n"                    /*PCH:PCL information from the stack */
                                                               /*Store the 24-bit result into 2x16-bit words */
                                                             /*in RAM at- MathErrInstructionAddrHigh:Low */
                        "mov     #_MathErrInstructionAddr, w1 \n"  /*Decrement that value by 2 if Accumulator based trap occurred*/
                        "clr     w2 \n"
                        "cp0     _divByZero \n"
                        "btsc    _SR, #1 \n"
                        "mov     #2, w2 \n"                  /*Store the 24-bit result into 2x16-bit words*/
                        "subr    w2, [w1], [w1++] \n"        /*in RAM at- AddrErrInstructionAddrHigh:Low*/
                        "clr     w2 \n"
                        "subbr   w2, [w1], [w1] \n"
                        "pop     w2 \n"
                        "pop.d   w0 ");
}

/*
EOF
*/


