/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
**********************************************************************/

#include "p30fxxxx.h"

#define DIVIDEBYZERO            /* <--If not defined then code example */
                                /* will mimic an accumulator overflow error */

/* Macros for Configuration Fuse Registers (copied from device header file):*/
_FOSC(CSW_FSCM_OFF & XT_PLL8);   /* Set up for canned oscillator/clock by 8x PLL */
                                /* Ensure FSCM and Clock-switching are ON */
_FWDT(WDT_OFF);                 /* Turn off the Watch-Dog Timer.  */
_FBORPOR(MCLR_EN & PWRT_OFF);   /* Enable MCLR reset pin and turn off the power-up timers. */
_FGS(CODE_PROT_OFF);            /* Disable Code Protection */

/* Global Variables and Functions */
int main (void);

int a = 10;
int b = 20;
int c = 0;

int main (void)
{
        INTCON1bits.OVATE = 1;  /* Enable Accumulator A Overflow traps */
        INTCON1bits.OVBTE = 1;  /* Enable Accumulator B Overflow traps */
        INTCON1bits.COVTE = 1;  /* Enable Catastrophic (i.e., permanent loss of sign bit)
                                /* Overflow traps for both accumulators*/
#ifdef  DIVIDEBYZERO
        a = b/c;                /* Perform a divide by zero operation */
#else
        asm volatile ("mov #0x7fff, w0"); /* Load 0x7FFF = Q15 fraction 0.999 into w0 */
        asm volatile ("lac w0, A");       /* Load w0 in Accumulator A (ACCAH word) */
        asm volatile ("lac w0, B");       /* Load w0 in Accumulator B (ACCBH word)*/
        asm volatile ("repeat #10000");   /* Repeat next instruction 10001 times*/
        asm volatile ("add A");           /* Add A and B and store results in A */
                                        /*This will cause an Accumulator Overflow error */
#endif
        while (1);              /* Loop endlessly*/

}


/*
EOF
*/


