/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        math_error_trap_30F_GP_MC_Sensor.s
* Dependencies:    Header (.inc) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* This routine applies for all dsPIC30F Motor Control, General Purpose
* and Sensor family devices.
**********************************************************************/

.include "p30fxxxx.inc"

.global __MathError
.global _accAOvfl
.global _accBOvfl
.global _divByZero
.global _MathErrInstructionAddrLow
.global _MathErrInstructionAddrHigh

.section .data
_accAOvfl: .hword 0x0000
_accBOvfl: .hword 0x0000
_divByZero: .hword 0x0000
_MathErrInstructionAddrLow: .hword 0x0000
_MathErrInstructionAddrHigh: .hword 0x0000

.section .text
__MathError:
        push.d  w0                      ;Save context of w0, w1 and w2
        push    w2

        mov     #0xFFFF, w0             ;Load an error flag value into w0

        btss    SR, #OA                 ;Test whether Math Error was caused by AccA Overflow
        bra     testOvflAccB            ;So we'll first check if Accumulator A Overflow Errors were
        btsc    INTCON1, #OVATE         ;enabled by the user to begin with. If neither OVATE
        bra     confirmedOvflAccA       ;nor COVTE is set then the user did not enable
        btss    INTCON1, #COVTE         ;Accumulator A overflow error traps in which case,
        bra     testOvflAccB            ;we'll check Accumulator B
confirmedOvflAccA:                      ;We are here if AccA overflow error traps were enabled
        clr     a                       ;For GP, MC and Sensor family dsPIC30F devices
        bclr    SR, #OA                 ;the condition causing the accumulator overflow error trap
        bclr    SR, #OAB                ;must be cleared prior to clearing the MATHERR flag
        mov     w0, _accAOvfl           ;Set a flag variable for the user
        bra     exit

testOvflAccB:
        btss    SR, #OB                 ;Test whether Math Error was caused by AccB Overflow
        bra     confirmedDivByZero      ;So we'll first check if Accumulator B Overflow Errors were
        btsc    INTCON1, #OVBTE         ;enabled by the user to begin with. If neither OVBTE
        bra     confirmedOvflAccB       ;nor COVTE is set then the user did not enable
        btss    INTCON1, #COVTE         ;Accumulator B overflow error traps in which case,
        bra     confirmedDivByZero      ;we know that the math error was caused by a divide by zero op
confirmedOvflAccB:
        clr     b                       ;We are here if AccB overflow error traps were enabled
        bclr    SR, #OB                 ;For GP, MC and Sensor family dsPIC30F devices
        bclr    SR, #OAB                ;the condition causing the accumulator overflow error trap
        mov     w0, _accBOvfl           ;must be cleared prior to clearing the MATHERR flag
        bra     exit                    ;Set a flag variable for the user

confirmedDivByZero:                     ;Math Error was caused by Divide by Zero error
        mov     w0, _divByZero
exit:
        bclr    INTCON1, #MATHERR       ;Clear the Math Error Trap

        ;Next we will estimate which instruction caused the the Math Error trap to take place
        mov     w15, w0                 ;Copy Stack Pointer (SP) into w0
        sub    w0, #6, w0               ;NOTE: Subtracting 6 from SP to estimate stack pointer
                                        ;location prior to 1 push.d and 1 push operation
        mov     #_MathErrInstructionAddrHigh, w1
        mov     [--w0], [w1--]          ;Pop the Program Counter (PC) from the stack
        mov     [--w0], [w1++]          ;Remember that the PC(PCH and PCL) is stacked
        bclr    [w1], #7                ;along with the SRL byte and IPL3 bit from
        inc     w1, w1                  ;CORCON. So we need to extract just the 24-bit
        clr.b   [w1]                    ;PCH:PCL information from the stack
                                        ;Store the 24-bit result into 2x16-bit words
                                        ;in RAM at- MathErrInstructionAddrHigh:Low
        mov     #_MathErrInstructionAddrLow, w1
        clr     w2                      ;Decrement the address value by 2 if Divide-by-Zero error occurred
        cp0     _divByZero              ;but
        btsc    SR, #Z
        mov     #2, w2                  ;Store the 24-bit result into 2x16-bit words
        subr    w2, [w1], [w1++]        ;in RAM at- AddrErrInstructionAddrHigh:Low
        clr     w2
        subbr   w2, [w1], [w1]
        pop     w2                      ;Restore w0, w1 and w2
        pop.d   w0

        retfie                          ;Return from interrupt


.end



