/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        clock_divide.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.32.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
**********************************************************************/

#include "p30fxxxx.h"
#include "OscillatorSystemCodes.h"

int clockDivide (unsigned int); //Subroutine to perform dynamic clock division operation


/*
Function:       int clockDivide (unsigned int)
Input:          int data type equal in value to 1, 4, 16 or 64
Return:         RET_CODE_OSC_SUCCESS for successful dynamic clock divide
                RET_CODE_OSC_ERROR for illegal clock divide request (i.e. values other than 1, 4, 16, 64)
Description:    This subroutine performs an OSCCONL unlock and then writes the
                correct requested clock divide value into the POST<1:0> bits in OSCCONL.
*/
int clockDivide (unsigned int dividerValue)
{
        int current_cpu_ipl;    /* declare temp variable for CPU IPL storing */
        char a, b, c, *p;
        b = OSCCONL_UNLOCK_CODE1;
        c = OSCCONL_UNLOCK_CODE2;
        p = (char *)&OSCCON;
        a = OSCCON;
        a = a & ZERO_MASK_FOR_POST_IN_OSCCON;

        switch (dividerValue)
        {
                case 1: a |= DIVIDE_BY_1 ;
                        break;
                case 4: a |= DIVIDE_BY_4 ;
                        break;
                case 16: a |= DIVIDE_BY_16;
                        break;
                case 64: a |= DIVIDE_BY_64;
                        break;
                default: return (RET_CODE_OSC_ERROR);
        }


        SET_AND_SAVE_CPU_IPL(current_cpu_ipl, 7);       /* Disable interrupts for unlock sequence below */

        /* Write Oscillator Post-scaler here low byte unlock sequence and enable LP Oscillator */
        asm volatile ("mov.b %1,[%0] \n"
                  "mov.b %2,[%0] \n"
                  "mov.b %3,[%0] \n" : /* no outputs */ : "r"(p), "r"(b), "r"(c),
                                                          "r"(a));

        /* restore CPU IPL value after executing unlock sequence */
        RESTORE_CPU_IPL(current_cpu_ipl);

        return RET_CODE_OSC_SUCCESS;
}

/*
EOF
*/


