/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        DCI_master_24bit_I2S_codec_interface.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.22 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
* HV               01/17/06  Bug fix to CSCKE/Add #ifdef 30F4013
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
**********************************************************************/

#include "p30fxxxx.h"

_FOSC(CSW_FSCM_OFF & XT_PLL16); /* Set up for XTxPLL16 mode since */
                                /* we will be tuning the FRC in this example */
_FWDT(WDT_OFF);                 /* Turn off the Watch-Dog Timer.  */
_FBORPOR(MCLR_EN & PWRT_OFF);   /* Enable MCLR reset pin and turn off the power-up timers. */
_FGS(CODE_PROT_OFF);            /* Disable Code Protection */

#define	FCY		29491200
#define	FS		48000
#define	FCSCK		64*FS
#define	LOOPBACKMODE	0

int main (void);
void DCIInit (void);
void __attribute__((__interrupt__)) _DCIInterrupt(void);

int temp = 0;

int main (void)
{
#if __dsPIC30F4013__
	ADPCFG |= 0x0F00;
#endif	
	DCIInit();
	while (1);
	return 0;
}

void DCIInit (void)
{

/*
In this section we will set up the DCI module for I2S operation to interface 
with a stereo audio codec sampling data at 48 KHz. The timing diagram is 
provided in Fig 1 below:
                                  FIGURE 1
       _______________________________
      |		 	    	      |	                              |
COFS: |	  	     	              |_______________________________|
       _   _   _   _   _   _   _   _   _   _   _   _   _   _   _   _   _    
CSCK:_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|
          |<--------Left Channel Data---->|<----Right Channel Data------->|
          |<---------32 bits------------->|<---------32 bits------------->|
          |<---TXBUF0---->|<----TXBUF1--->|<---TXBUF2---->|<----TXBUF3--->|
          |<---RXBUF0---->|<----RXBUF1--->|<---RXBUF2---->|<----RXBUF3--->|
          |<--TimeSlot0-->|<--TimeSlot1-->|<--TimeSlot0-->|<--TimeSlot1-->|
          |<--------------------1/Fs = 20.8 microseconds----------------->|
*/
/*
DCI Control Register DCICON1 Initialization
*/
	DCICON1bits.CSCKD = 0;	/*Serial Bit Clock (CSCK pin) is output	*/
	DCICON1bits.CSCKE = 1;	/*Data changes on falling edge		*/
				/*& sampled on rising edge of CSCK	*/
	DCICON1bits.COFSD = 0;	/*Frame Sync Signal is output		*/
	DCICON1bits.UNFM = 0;	/*If Tx Buffers are not reloaded after	*/
				/*transmission is complete, then 0's are*/
				/*transmitted				*/
	DCICON1bits.CSDOM = 0;	/*CSDO pin is held low during inactive 	*/
				/*time-slots				*/
	DCICON1bits.DJST = 0;	/*Data is transmitted one bit clock	*/
				/*after Frame Sync pulse		*/
	DCICON1bits.COFSM = 1;	/*Frame Sync Signal set up for I2S mode */

#if	LOOPBACKMODE
	DCICON1bits.DLOOP = 1;	/*Set up Digital Loop-back from CSDO to */
				/*CSDI pins*/
#endif

/*
DCI Control Register DCICON2 Initialization
*/
/* 
In this section we will set up the frame length for
the operation shown in Figure 1.
*/

	DCICON2bits.BLEN = 3;	/*Enable all 4 transmit & receive buffer*/
				/*for use				*/
	DCICON2bits.COFSG = 1;	/*Set up Frame Sync Generator for 2	*/
				/*words per frame			*/
	DCICON2bits.WS = 15;	/*Set up Word Size to 16 bits		*/

/*
DCI Control Register DCICON3 Initialization
*/
	DCICON3 = FCY / (2*FCSCK - 1);	/*Set up CSCK Bit Clock Frequency*/

/*
Transmit Slot Control Register Initialization
*/
	TSCONbits.TSE0 = 1;	/*Transmit on Time Slot 0		*/
	TSCONbits.TSE1 = 1;	/*Transmit on Time Slot 1		*/
				/*2 timeslots are enabled to send 32-   */
				/*bit data per channel (left or right)  */

/*
Receiver Slot Control Register Initialization
*/
	RSCONbits.RSE0 = 1;	/*Receive on Time Slot 0		*/
	RSCONbits.RSE1 = 1;	/*Receive on Time Slot 1		*/
				/*2 timeslots are enabled to send 32-   */
				/*bit data per channel (left or right)  */


	_DCIIE = 1;		/*Clear the DCI Interrupt Flag          */
	_DCIIF = 0;		/*Enable DCI ISR processing             */
	DCICON1bits.DCIEN = 1;  /*Enable the DCI module                 */

}

void __attribute__((__interrupt__)) _DCIInterrupt(void)
{
	temp = RXBUF0;
	temp = RXBUF1;
	temp = RXBUF2;
	temp = RXBUF3;
	TXBUF0 = 0xAAAA;	/*In this example, we will simply copy  */
	TXBUF1 = 0x0001;	/*data received to a temporary variable */
	TXBUF2 = 0x0002;	/*and send some known data out on the   */
	TXBUF3 = 0x0003;	/*CSDO pin so we can watch this on an oscilloscope */
}

/*
EOF
*/


